<?php

namespace FiloBlu\Flow\Console\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class CriticalCommand
 * @package FiloBlu\Flow\Console\Command
 */
class CheckIndexersCommand extends Command
{
    /**
     * @var \Magento\Framework\App\State
     */
    protected $_state;

    /**
     * @var \FiloBlu\Flow\Model\Processor
     */
    protected $_processor;

    /**
     * @var \FiloBlu\Flow\Helper\System
     */
    protected $_systemHelper;

    /**
     * FetchCommand constructor.
     * @param \Magento\Framework\App\State $state
     */
    public function __construct(
        \Magento\Framework\App\State $state,
        \FiloBlu\Flow\Model\Processor $processor,
        \FiloBlu\Flow\Helper\System $systemHelper

    ) {
        $this->_state = $state;
        $this->_systemHelper = $systemHelper;
        $this->_processor = $processor;

        parent::__construct();
    }

    /**
     *
     */
    protected function configure()
    {
        $this->setName('flow:check:indexer')
            ->setDescription('Check indexers to enable');
    }

    /**
     * @param InputInterface $input
     * @param OutputInterface $output
     * @return int|null|void
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        try {
            $this->state->setAreaCode(\Magento\Framework\App\Area::AREA_ADMINHTML);
        } catch (\Exception $e) {
            $output->writeln("Areacode was already set, skipping!");
        }

        if ($this->_processor->canEnableIndexers()) {
            $this->_systemHelper->enableIndexers();
            $output->writeln('<info>Indexers enabled !</info>');

        } else {
            $output->writeln('<info>No indexer to enable !</info>');
        }
    }

}