<?php
/**
 * Copyright Â© 2015 FiloBlu . All rights reserved.
 */

namespace FiloBlu\Flow\Helper;

/**
 * Class Product
 * @package FiloBlu\Flow\Helper
 */
class Product extends \Magento\Framework\App\Helper\AbstractHelper
{
    const CONFIG_FLOW_ATTRIBUTE_PATH = 'filoblu_flow/flow_process/flow_attribute_select';
    const CONFIG_FLOW_ONLYCONFIGURABLE_PATH = 'filoblu_flow/flow_process/flow_only_configurable_category';
    const CONFIG_FLOW_UPDATECATEGORY_PATH = 'filoblu_flow/flow_process/flow_update_category';
    const CONFIG_FLOW_MERGE_PATH = 'filoblu_flow/flow_process/flow_merge_category';
    const CONFIG_FLOW_ASSIGN_ROOT_CAT = 'filoblu_flow/flow_process/flow_assign_root_categories';
    const CONFIG_FLOW_ASSIGN_FULL_TREE_PATH = 'filoblu_flow/flow_process/flow_assign_full_tree_category';
    const CONFIG_FLOW_UPDATE_CONFIGURABLE_URL = 'filoblu_flow/flow_process/flow_configurable_url';
    const CONFIG_FLOW_ADD_TO_ALL_WEBSITES = 'filoblu_flow/flow_ite/add_to_websites';
    const CONFIG_FLOW_ITE_COPY_NAME_TO_DESCRIPTION = 'filoblu_flow/flow_ite/copy_name_in_description';
    const CONFIG_FLOW_ITE_COPY_NAME_TO_SHORT_DESCRIPTION = 'filoblu_flow/flow_ite/copy_name_in_short_description';
    const CONFIG_FLOW_ITE_DISABLE_SHOP_STATUS = 'filoblu_flow/flow_ite/disable_shop_status';
    const CONFIG_FLOW_ITE_HIDE_PRICE_STATUS = 'filoblu_flow/flow_ite/hide_price_status';
    const CONFIG_FLOW_INHERITCATOFSIMPLE_PATH = 'filoblu_flow/flow_process/flow_inherit_categories_of_simple';
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;
    /**
     * @var \Magento\Framework\App\ProductMetadataInterface
     */
    protected $_productMetadata;
    /**
     * @var \Magento\Framework\App\ResourceConnection
     */
    protected $_resource;
    /**
     * @var
     */
    protected $connection;
    /**
     * @var \Monolog\Logger|\Monolog\Logger
     */
    protected $_logger;
    protected $_productRepository;

    /**
     * @var \Magento\Framework\App\ObjectManager
     */
    protected $_objectManager;

    /**
     * @var \Magento\Framework\Module\Manager
     */
    protected $_moduleManager;
    /**
     * Product constructor.
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\App\ProductMetadataInterface $_productMetadata
     * @param \Magento\Framework\App\ResourceConnection $resource
     * @param LoggerProvider $loggerProvider
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\App\ProductMetadataInterface $_productMetadata,
        \Magento\Framework\App\ResourceConnection $resource,
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \Magento\Catalog\Model\ProductRepository $productRepository,
        \Magento\Framework\Module\Manager $moduleManager
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->_resource = $resource;
        $this->_productMetadata = $_productMetadata;
        $this->_logger = $loggerProvider->getLogger();
        $this->_productRepository = $productRepository;
        $this->_moduleManager = $moduleManager;

        $this->_objectManager = \Magento\Framework\App\ObjectManager::getInstance();
    }

    // Import categories only on configurables?
    public function getOnlyConfigurableCategoryFlag()
    {

        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        $flag = $this->scopeConfig->getValue(self::CONFIG_FLOW_ONLYCONFIGURABLE_PATH, $storeScope);

        return $flag;
    }

    // If a simple is passed do i need to set the same categories on parents?
    public function getParentInheritCategoriesOfSimple()
    {
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        $flag = $this->scopeConfig->getValue(self::CONFIG_FLOW_INHERITCATOFSIMPLE_PATH, $storeScope);

        return $flag;
    }

    // Is process categories enabled?
    public function processCategoriesEnabledFlag()
    {
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        $flag = $this->scopeConfig->getValue(self::CONFIG_FLOW_UPDATECATEGORY_PATH, $storeScope);

        return $flag;
    }

    // Categories need to be merged with the old ones?
    public function mergeCategoriesFlag()
    {
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        $flag = $this->scopeConfig->getValue(self::CONFIG_FLOW_MERGE_PATH, $storeScope);

        return $flag;
    }

    // Assign product also on root categories? (level 1 categories on database)
    public function assignRootCategories()
    {
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        $flag = $this->scopeConfig->getValue(self::CONFIG_FLOW_ASSIGN_ROOT_CAT, $storeScope);

        return $flag;
    }

    // Do I need to assign the product to the full category tree?
    public function assignFullTree()
    {
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        $flag = $this->scopeConfig->getValue(self::CONFIG_FLOW_ASSIGN_FULL_TREE_PATH, $storeScope);

        return $flag;
    }

    /**
     * @return mixed
     */
    public function getUpdateConfigurableUrl()
    {
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        $flag = $this->scopeConfig->getValue(self::CONFIG_FLOW_UPDATE_CONFIGURABLE_URL, $storeScope);

        return $flag;
    }

    /**
     * @return mixed
     */
    public function getCopyNameInDescription()
    {
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        $flag = $this->scopeConfig->getValue(self::CONFIG_FLOW_ITE_COPY_NAME_TO_DESCRIPTION, $storeScope);

        return $flag;
    }

    /**
     * @return mixed
     */
    public function getCopyNameInShortDescription()
    {
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        $flag = $this->scopeConfig->getValue(self::CONFIG_FLOW_ITE_COPY_NAME_TO_SHORT_DESCRIPTION, $storeScope);

        return $flag;
    }

    /**
     * @return mixed
     */
    public function isDisableShopModuleEnabled(){
        return $this->_moduleManager->isEnabled('FiloBlu_DisableShop');
    }

    /**
     * @return mixed
     */
    public function getDisableShopStatus(){
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        $flag = $this->scopeConfig->getValue(self::CONFIG_FLOW_ITE_DISABLE_SHOP_STATUS, $storeScope);

        return $flag;
    }

    /**
     * @return mixed
     */
    public function getHidePriceStatus(){
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        $flag = $this->scopeConfig->getValue(self::CONFIG_FLOW_ITE_HIDE_PRICE_STATUS, $storeScope);

        return $flag;
    }

    /**
     * @return int
     * @throws \Exception
     * @deprecated 
     */
    public function getProductAttributeSet()
    {
        throw new \Exception(__CLASS__.'::'.__METHOD__.' Avoid using this deprecated method');
        return 4;
    }

    /**
     * @return bool
     */
    public function createNewProducts()
    {
        return true;
    }

    /**
     * @return int
     */
    public function getInitialProductStatus()
    {
        return \Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_DISABLED;
    }

    /**
     * @return mixed
     */
    public function getAddToWebsites()
    {
        return $this->scopeConfig->getValue(self::CONFIG_FLOW_ADD_TO_ALL_WEBSITES);
    }

    /**
     * @param $TaxClassCode
     *
     * @return int
     * @throws \Exception
     * @deprecated
     */
    public function getMappedTaxClassId($TaxClassCode)
    {
        throw new \Exception(__CLASS__.'::'.__METHOD__.' Avoid using this deprecated method');
        return 2;
    }

    /**
     * @param $childProducts
     * @return mixed
     * @throws \Exception
     */
    public function getParentProductsIds($childProducts)
    {
        $relationTableName = $this->_objectManager->get('\Magento\Framework\App\ResourceConnection')->getTableName('catalog_product_super_link');

        $products = $this->_objectManager->create('Magento\Catalog\Model\ResourceModel\Product\Collection');

        $products->getSelect()->join(array('super_link' => $relationTableName), 'e.entity_id = super_link.parent_id');
        $products->getSelect()->where('super_link.product_id IN (?)', $childProducts);
        //$products->addFieldToFilter('type_id', $productType);
        $products->getSelect()->group('e.entity_id');

        $result = $products->getAllIds();

        // Need to convert row_id into entity_id
        if ($this->isEnterpriseEdition() && count($result)) {
            foreach ($result as $key => $value) {

                if (!$value) {
                    continue;
                }

                try {
                    $entity_id = $this->getConnection()->fetchRow("SELECT entity_id FROM catalog_product_entity WHERE row_id = ?",
                        [$value]);
                } catch (\Exception $e) {
                    $message = "Can't find entity_id from row_id '{$value}')";
                    $this->_logger->info($message);
                    throw new \Exception($e->getMessage());
                }

                if (isset($entity_id['entity_id']) && $entity_id['entity_id'] > 0) {
                    $result[$key] = $entity_id['entity_id'];
                }

            }
        }

        return $result;
    }

    /**
     * @return bool
     */
    protected function isEnterpriseEdition()
    {
        $edition = $this->_productMetadata->getEdition();
        return ($edition == 'Enterprise') ? true : false;

    }

    /**
     * @return \Magento\Framework\DB\Adapter\AdapterInterface
     */
    protected function getConnection()
    {
        if (!$this->connection) {
            $this->connection = $this->_resource->getConnection('core_read');
        }
        return $this->connection;
    }

    /**
     * @return array
     */
    public function getSimpleToConfigurableProductAttributesToExclude()
    {
        $saved = [];

        if (is_string($saved)) {
            return explode(',', $saved);
        }

        return $saved;
    }

    /**
     * @return array
     */
    public function getSimpleToConfigurableSyncAttributes()
    {

        /*
        $attributesToCopy = \Magento\Framework\App\ObjectManager::getInstance()->create('FiloBlu\Flow\Model\System\ProductAttributes')->getAttributesToCopy()->getColumnValues('attribute_code');
        $excludedAttributes = $this->getSimpleToConfigurableProductAttributesToExclude();
        $attributesToCopy = array_diff($attributesToCopy, $excludedAttributes);
        */

        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;

        $attributesToCopy = $this->scopeConfig->getValue(self::CONFIG_FLOW_ATTRIBUTE_PATH,
            $storeScope);//array("name", "description", "short_description");

        if (!empty($attributesToCopy)) {
            return explode(",", $attributesToCopy);
        }

        return [];
    }

    /**
     * @return bool
     */
    public function getUpdateConfigurablePrice()
    {
        return false;
    }

    /**
     * Get Configurable Childs by Sku
     *
     * @param $sku
     *
     * @return array
     */
    public function getConfigurableChildsBySku($sku){
        $simples = [];

        try {
            // Get Product
            $product = $this->_productRepository->get($sku);

            //If product is configurable
            if($product->getTypeId() == 'configurable'){
                $attributes = $product->getTypeInstance()->getConfigurableOptions($product);
                //Get childs for super attribute
                foreach($attributes as $attribute) {
                    foreach ($attribute as $child){
                        $simples[] = $child['sku'];
                    }
                }
            }
            //if is simple
            else {
                $simples[] = $sku;
            }
        }
        catch (\Exception $e) {
            $simples = [];
            $this->_logger->info($e->getMessage()." for sku ".$sku);
        }

        return $simples;
    }
}